<?php
/**
 * Core Design Attachments plugin for Joomla! 1.5
 * @author		Daniel Rataj, <info@greatjoomla.com>
 * @package		Joomla
 * @subpackage	Content
 * @category	Plugin
 * @version		1.0.0 Experimental
 * @copyright	Copyright (C) 2007 - 2008 Core Design, http://www.greatjoomla.com
 * @license		http://www.gnu.org/licenses/gpl.html GNU/GPL
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

// Import library dependencies
jimport('joomla.plugin.plugin');
jimport('joomla.filesystem.file');
jimport('joomla.filesystem.path');
jimport('joomla.filesystem.folder');

/**
 * Core Design Attachments plugin
 *
 * @author		Daniel Rataj <info@greatjoomla.com>
 * @package		Core Design
 * @subpackage	System
 */
class plgContentCdAttachments extends JPlugin
{
	/**
	 * Constructor
	 *
	 * For php4 compatability we must not use the __constructor as a constructor for plugins
	 * because func_get_args ( void ) returns a copy of all passed arguments NOT references.
	 * This causes problems with cross-referencing necessary for the observer design pattern.
	 *
	 * @access	protected
	 * @param	object		$subject The object to observe
	 * @since	1.0
	 */
	function plgContentCdAttachments(&$subject)
	{
		parent::__construct($subject);

		// load plugin parameters
		$this->plugin = &JPluginHelper::getPlugin('content', 'cdattachments');
		$this->params = new JParameter($this->plugin->params);

		// load language
		JPlugin::loadLanguage('plg_content_cdattachments', JPATH_ADMINISTRATOR);
	}

	/*
	 * Joomla! onAfterDispatch() function
	 */
	function onAfterDispatch() {

		$task = JRequest::getCmd('attachments', '', 'post');

		switch ($task) {
			case 'getItems':
				$this->close($this->previewTemplate($this->getItems()));
				break;
			case 'download':
				$this->download();
				break;
			case 'fileTemplate':
				$tmpl = $this->fileTemplate();
				$this->close($tmpl);
				break;
			case 'addAttachment':
				$this->addAttachment();
				break;
			case 'editAttachment':
				$this->editAttachment();
				break;
			case 'deleteAttachment':
				$this->close($this->deleteAttachment());
				break;
			case 'filemgmt':
				$this->close($this->filemgmt());
				break;
			case 'uploadFile':
				$this->uploadFile();
				break;
			case 'removeFile':
				$this->close($this->removeFile());
				break;
			default:
				return;
				break;
		}

	}

	/**
	 * Joomla! onPrepareContent() function
	 */
	function onPrepareContent(&$article, &$params, $limitstart=0)
	{
		$document = &JFactory::getDocument(); // set document for next usage

		if ($document->getType() != 'html') return; // prevent script loading in PDF or RSS output

		// disable category and section view
		if (JRequest::getCmd('view', '') == 'section' or JRequest::getCmd('view', '') == 'category') {
			$layout = JRequest::getCmd('layout');
			if ($layout == 'blog') {
				if ($this->params->get('disable_sectionview', 0)) return;
				if ($this->params->get('disable_categoryview', 0)) return;
			} else {
				return;
			}
		}
		
		// disable section, category or article if required
		jimport('joomla.utilities.arrayhelper');

		$disablesection = explode(',', trim($this->params->get('disablesection' , '')));
		JArrayHelper::toInteger($disablesection);

		$disablecategory = explode(',', trim($this->params->get('disablecategory' , '')));
		JArrayHelper::toInteger($disablecategory);

		$disablearticle = explode(',', trim($this->params->get('disablearticle' , '')));
		JArrayHelper::toInteger($disablearticle);

		if (in_array($article->sectionid, $disablesection)) return;
		if (in_array($article->catid, $disablecategory)) return;
		if (in_array($article->id, $disablearticle)) return;
		
		$this->levelList();
		
		// check Scriptegrator
		if (!defined('_JSCRIPTEGRATOR')) {
			JError::raiseNotice('', JText::_('CDATTACHMENTS_ENABLE_SCRIPTEGRATOR'));
			return;
		}

		// requires Scriptegrator version 1.3.5 or higher
		$version = '1.3.5';
		if (!JScriptegrator::versionRequire($version)) {
			JError::raiseNotice('', JText::sprintf('CDATTACHMENTS_SCRIPTEGRATOR_REQUIREVERSION', $version));
			return;
		}

		// check jQuery
		if (!JScriptegrator::checkLibrary('jquery', 'site'))
		{
			JError::raiseNotice('', JText::_('CDATTACHMENTS_MISSING_JQUERY'));
			return;
		}
	  
		// check attachment directory
		$checkDir = $this->checkDir();
		if ($checkDir) {
			JError::raiseNotice('', JText::_($checkDir));
			return;
		}
	  
		// check if db table exists
		$this->checkDatabase();
	  
		$live_path = JURI::root(true) . '/'; // define live site

		$compress = 1;
		$theme = $this->params->get('theme', 'smoothness');

		static $once = false;
		if (!$once) {
				
			// add CSS stylesheet and JavaScript file
			$document->addStyleSheet( $live_path . 'plugins/content/cdattachments/css/cdattachments.css', 'text/css' );
			//$document->addScript($live_path . 'plugins/content/cdattachments/utils/js/jquery.attachments.min.js');
			$document->addScript($live_path . 'plugins/content/cdattachments/utils/js/jquery.attachments.js');
				
			JScriptegrator::UILoader($compress, 'ui.dialog');
			
			JScriptegrator::UICssLoader($compress, $theme, 'ui.core');
			JScriptegrator::UICssLoader($compress, $theme, 'ui.theme');
			JScriptegrator::UICssLoader($compress, $theme, 'ui.dialog');
			
				
			// set language strings
			$language_strings = array();
			$language_strings []= 'CDATTACHMENTS';
			$language_strings []= 'CDATTACHMENTS_CLOSE';
			$language_strings []= 'CDATTACHMENTS_ADDNEW';
			$language_strings []= 'CDATTACHMENTS_BACK';
			$language_strings []= 'CDATTACHMENTS_EMPTY_VALUE';
			$language_strings []= 'CDATTACHMENTS_FILE_IS_PROTECTED';
			$language_strings []= 'CDATTACHMENTS_FILEINFORMATION';
			$language_strings []= 'CDATTACHMENTS_REALLY';
			$language_strings []= 'CDATTACHMENTS_SAVED';
			$language_strings []= 'CDATTACHMENTS_EDITED';
			$language_strings []= 'CDATTACHMENTS_DELETED';
			$language_strings []= 'CDATTACHMENTS_FILES';
			$language_strings []= 'CDATTACHMENTS_SELECT_FILE';
			$this->languageStrings($language_strings);
				
			// set option strings
			$option_strings = array();
			$option_strings []= 'token' . '|' . $this->getToken();
			$option_strings []= 'theme' . '|' . $theme;
			$option_strings []= 'count' . '|' . str_replace(array('(', ')'), '', $this->attachmentsCount($article->id));
			$option_strings []= 'addnewauth' . '|' . ($this->hasEditPermission() ? 1 : 0);
			$option_strings []= 'filemanagerauth' . '|' . ($this->filemanagerPermission() ? 1 : 0);
				
			$this->optionStrings($option_strings);
				
			$init_js = "
			<script type=\"text/javascript\">
	        <!--
			jQuery(document).ready(function($){
				// init Attachments application
				if ($.attachments) {
					$.attachments.initiator();
				}
			});
			// -->
			</script>";
			$document->addCustomTag($init_js);
				
			$once = true;
		}

		$button_attachments = '<div class="attachments ' . $theme . '"><button class="loadattachments" id="' . $article->id . '" title="' . JText::_('CDATTACHMENTS') . '" type="submit">' . JText::_('CDATTACHMENTS') . ' (<span>' . JText::sprintf('CDATTACHMENTS_COUNT', $this->attachmentsCount($article->id)) . '</span>)</button></div>';

		$regex = '#{attachments}#is';

		if (preg_match($regex, $article->text)) {
			return $article->text = preg_replace($regex, $button_attachments, $article->text);
		}

		// return button
		return $article->text .= $button_attachments;

	}

	/**
	 * Method to determine a hash for anti-spoofing variable names
	 *
	 * @return	string	Hashed var name
	 */
	function getToken() {
		jimport('joomla.utilities.utility');
		$token = JUtility::getToken();
		return $token;
	}

	/**
	 * Language strings
	 *
	 * Insert language strings to header
	 *
	 * @param $languages	language array
	 * @return void
	 */
	function languageStrings($languages = array()) {

		if (!is_array($languages) or empty($languages)) {
			return;
		}

		$lang = '';

		$lang .= "<script type=\"text/javascript\">\n";
		$lang .= "<!--\n";
		$lang .= "// Attachments language strings\n";

		$lang .= "jQuery(document).ready(function($) {";
		$lang .="\n";
		$lang .="if ($.attachments) {";
		$lang .="\n";
		$lang .= "	$.extend( $.attachments, { language: {";
		$lang .="\n";

		foreach ($languages as $language) {
			if (JText::_($language) == $language) {
				$language = '';
			}
			if ($language) {
				$lang .= '		' . $language . ' : "' . JText::_($language) . '",';
				$lang .="\n";

			}
		}
		$lang = preg_replace('#,\n$#', '', $lang); // remove "," from end
		$lang .="\n";
		$lang .= "	}});";
		$lang .="\n";
		$lang .= "};";
		$lang .="\n";
		$lang .="});";
		$lang .="\n";
		$lang .= "// -->\n";
		$lang .= "</script>\n";

		$document = &JFactory::getDocument(); // set document for next usage
		$document->addCustomTag($lang);
	}

	/**
	 * Option strings
	 *
	 * Insert option strings to header
	 *
	 * @param $option_array		option array
	 * @return void
	 */
	function optionStrings($option_array = array()) {
		if (!is_array($option_array) or empty($option_array)) {
			return;
		}

		$options = '';

		$options .= "<script type=\"text/javascript\">\n";
		$options .= "<!--\n";
		$options .= "// Attachments settings\n";

		$options .= "jQuery(document).ready(function($) {";
		$options .="\n";
		$options .="if ($.attachments) {";
		$options .="\n";
		$options .= "	$.extend( $.attachments, { settings: {";
		$options .="\n";

		foreach ($option_array as $option) {
			$string = explode('|', $option);
			$option_name = ( isset($string[0]) ? $string[0] : '' );
			$value = ( isset($string[1]) ? $string[1] : '' );

			$options .= '		' . $option_name . ' : "' . $value . '",';
			$options .="\n";
		}

		$options = preg_replace('#,\n$#', '', $options); // remove "," from end
		$options .="\n";
		$options .= "	}});";
		$options .="\n";
		$options .= "};";
		$options .="\n";
		$options .="});";
		$options .="\n";
		$options .= "// -->\n";
		$options .= "</script>\n";

		$document = &JFactory::getDocument(); // set document for next usage
		$document->addCustomTag($options);
	}

	/**
	 * Check database
	 *
	 * Check if database exists, if no, create one
	 *
	 * @return void
	 */
	function checkDatabase() {
		$db = &JFactory::getDBO();

		$query = '
	  CREATE TABLE IF NOT EXISTS `#__cdattachments` (' .
            '`id` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,' .
        	'`articleid` INT(11) UNSIGNED NOT NULL,' .
        	'`userid` INT(11) UNSIGNED NOT NULL,' .
        	'`file` VARCHAR(255) NOT NULL DEFAULT \'\',' .
			'`title` VARCHAR(255) NOT NULL DEFAULT \'\',' .
        	'`desc` MEDIUMTEXT DEFAULT \'\',' .
        	'`type` VARCHAR(255) NOT NULL DEFAULT \'\',' .
        	'`size` INT(11) UNSIGNED NOT NULL DEFAULT 0,' .
            '`access` INT(11) UNSIGNED NOT NULL DEFAULT 0,' .
        	'`password` VARCHAR(255) DEFAULT \'\',' .
        	'`published` INT(11) UNSIGNED NOT NULL DEFAULT 0,' .
        	'`created` DATETIME DEFAULT NULL,' .
        	'`modified` DATETIME DEFAULT NULL,' .
			'`modifiedby` INT(11) UNSIGNED NOT NULL,' .
        	'`downloaded` INT(11) UNSIGNED NOT NULL DEFAULT 0,' .
			' PRIMARY KEY (`id`)' .
            ') TYPE=MyISAM;';

		$db->setQuery($query);

		if (!$db->query())
		{
			JError::raiseError(500, $db->stderr());
		}
	}

	/**
	 * Check attachment directory
	 *
	 * Check directory routine
	 *
	 * @return string
	 */
	function checkDir() {

		$dir = $this->attachmentsDir();

		// check if dir exists
		if (!JFolder::exists($dir)) {
			return JText::_('CDATTACHMENTS_CHECK_DIR');
		}

		// write .htaccess file
		if (!JFile::write($dir . DS . '.htaccess', 'deny from all')) {
			return JText::_('CDATTACHMENTS_DIR_WRITEABLE');
		}

		// write blank index.htnl
		if (!JFile::write($dir . DS . 'index.html', '<html><body bgcolor="#FFFFFF"></body></html>')) {
			return JText::_('CDATTACHMENTS_DIR_WRITEABLE');
		}

		return;

	}

	/**
	 * Count of messages
	 *
	 * Return count of read and unread messages
	 *
	 * @return int
	 */
	function attachmentsCount($article_id = 0) {

		$user	=& JFactory::getUser();

		$db = &JFactory::getDBO();
		$articleid = ($article_id ? $article_id : 0);
		$published = 1;
		$access = $user->gid;

		$query = 'SELECT `id` FROM `#__cdattachments` WHERE `articleid` = ' . $articleid . ' AND `published` = ' . $published . ' AND `access` <= ' . $access . '';

		$db->setQuery($query);
		$rows = count($db->loadObjectList());

		return $rows;
	}

	/**
	 * Global close wrapper
	 *
	 * @param $msg		message to display
	 *
	 * @return void
	 */
	function close($msg = '') {
		global $mainframe;
		$mainframe->close($msg);
	}

	/**
	 * Load attachments
	 *
	 * Load attachments from database
	 *
	 * @return object
	 */
	function getItems() {
		$user	=& JFactory::getUser();

		$db = &JFactory::getDBO();
		$articleid = JRequest::getInt('articleid', 0, 'post');
		$published = 1;
		$access = $user->gid;

		$query = 'SELECT * FROM `#__cdattachments` WHERE `articleid` = ' . $articleid . ' AND `published` = ' . $published . ' AND `access` <= ' . $access . '';

		$db->setQuery($query);
		$rows = $db->loadObjectList();
		return $rows;
	}

	/**
	 * Run template
	 *
	 * Put database object list to the template.�
	 *
	 * @param	$object		file information
	 * @return	string
	 */
	function previewTemplate($object) {

		// Check for request forgeries
		JRequest::checkToken() or jexit( 'Invalid Token' );

		$tmpl = '';
		$count = count($object);

		if ($count) {
			foreach ($object as $key => $obj) {
				$tmpl .= '<div class="document">';
				
				$tmpl .= '<div class="downloadbutton">';
				$tmpl .= '<form action="" name="downloadfile" method="post">';
				$tmpl .= '<button type="submit" title="' . JText::_('CDATTACHMENTS_DOWNLOAD') . '"' . (JFile::exists($this->attachmentsDir() . DS . $obj->file) ? '' : ' disabled="true"') . '>' . JText::_('CDATTACHMENTS_DOWNLOAD') . '</button>';
				$tmpl .= '<input type="hidden" name="fileid" value="' . $obj->id . '" />';
				if ($obj->password) $tmpl .= '<input type="hidden" name="filepassword" value="" />';
				$tmpl .= '<input type="hidden" name="attachments" value="download" />';
				$tmpl .= JHTML::_( 'form.token' );
				$tmpl .= '</form>';
				$tmpl .= '</div>';

				$tmpl .= '<div>';
					
				if ($this->params->get('fileinformation', 0)) {
					$fileinfo = true;
				} else {
					$fileinfo = false;
				}
					
				$tmpl .= '<div class="title">' . $obj->title;
				$tmpl .= ($fileinfo ? ' <span class="smallinfo">' . ($this->params->get('fileinformation_size', 0) ? JText::sprintf('CDATTACHMENTS_FILESIZE', $this->byte_convert($obj->size)) . ', ' : '') . ($this->params->get('fileinformation_hits', 0) ? JText::sprintf('CDATTACHMENTS_HITS', $obj->downloaded) : '') . '</span>' : '');
				$tmpl .= '</div>';
				if ($obj->desc) $tmpl .= '<div class="desc">' . $obj->desc . '</div>';
				else $tmpl .= '<div class="desc">&nbsp;</div>';
				$tmpl .= '</div>';
					
				$dateformat = JText::_($this->params->get('dateformat', 'DATE_FORMAT_LC2'));
					
				$tmpl .= '<div class="fileicons">';
				if ($fileinfo) $tmpl .= '<div class="fileiconinfo" title="' . JText::_('CDATTACHMENTS_FILEINFORMATION') . '"></div>';
				if ($obj->password) $tmpl .= '<div class="fileiconlocked" title="' . JText::_('CDATTACHMENTS_PASSWORD_REQUIRED') . '"></div>';
				if ($this->hasEditPermission($obj->id)) $tmpl .= '<div class="fileicondelete" title="' . JText::_('CDATTACHMENTS_DELETE') . '"></div>';
				if ($this->hasEditPermission($obj->id)) $tmpl .= '<div class="fileiconedit" title="' . JText::_('CDATTACHMENTS_EDIT') . '"></div>';

				if ($fileinfo) {
					$tmpl .= '<span class="uploadedinfo">';
					if ($this->params->get('fileinformation_uploadedby', 1)) $tmpl .= '' . JText::sprintf('CDATTACHMENTS_CREATEDBBY', $this->getUsername($obj->userid));
					if ($this->params->get('fileinformation_created', 1)) $tmpl .= ($this->params->get('fileinformation_uploadedby', 0) ? ' (' : '') . JHTML::_('date', $obj->created, $dateformat) . ($this->params->get('fileinformation_uploadedby', 0) ? ')' : '');
					$tmpl .= '</span>';
				}

				$tmpl .= '<div class="cleaner"></div>';
				$tmpl .= '</div>';
					
				if ($fileinfo) {
					$tmpl .= '<div class="fileinformation">';
					if ($this->params->get('fileinformation_name', 0)) $tmpl .= JText::sprintf('CDATTACHMENTS_FILENAME', $obj->file) . '<br />';
					if ($this->params->get('fileinformation_type', 0)) $tmpl .= JText::sprintf('CDATTACHMENTS_FILETYPE', $obj->type) . '<br />';
					if ($this->params->get('fileinformation_modified', 0) and $obj->created != $obj->modified) {
						$tmpl .= JText::sprintf('CDATTACHMENTS_MODIFIED', JHTML::_('date', $obj->modified, $dateformat), $this->getUsername($obj->modifiedby)) . '<br />';
					}
					$tmpl .= '</div>';
				}
					
				$tmpl .= '</div>';

				if (($count - $key) > 1) {
					$tmpl .= '<div class="spacer"></div>';
				}

			}
		} else {
			$tmpl .= JText::_('CDATTACHMENTS_NO_ATTACHMENTS');
		}

		return $tmpl;
	}

	/**
	 * Get username via document owner
	 *
	 * @param $userid	user ID
	 *
	 * @return string	username
	 */
	function getUsername($userid = 0) {
		$user =& JFactory::getUser($userid);
		return $user->username;
	}

	/**
	 * List attachment directory
	 *
	 * @return string	Select list
	 */
	function selectFile($file = '') {

		$files = JFolder::files($this->attachmentsDir(), '.', false, false, array('.htaccess', 'index.html'));

		$selectlist = '<select name="file" id="selectfile" size="1" title="' . JText::_('CDATTACHMENTS_SELECTFILE') . '">';
		$selectlist .= '<option value="">' . JText::_('CDATTACHMENTS_SELECTFILE') . '</option>';
		foreach ($files as $files_item) {
			if ($files_item == $file) {
				$selectlist .= '<option value="' . $files_item . '" selected>' . $files_item . '</option>';
			} else {
				$selectlist .= '<option value="' . $files_item . '">' . $files_item . '</option>';
			}
		}
		$selectlist .= '</select>';

		return $selectlist;
	}

	/**
	 * Template to add new item
	 *
	 * @return string
	 */

	function fileTemplate() {

		// Check for request forgeries
		JRequest::checkToken() or jexit( 'Invalid Token' );

		if (!$this->hasEditPermission()) {
			return JText::_('CDATTACHMENTS_NOT_ALLOWED');
		}
		
		if(!JFolder::files($this->attachmentsDir(), '.', false, false, array('.htaccess', 'index.html', '.svn', 'CVS'))) return JText::_('CDATTACHMENTS_NO_FILES');

		$fileid = JRequest::getInt('fileid', 0, 'post');
		$mode = ($fileid ? 'editAttachment' : 'addAttachment');

		$fileinfo = $this->dbFileInfo('id', $fileid);

		$action_url = $this->url();

		$levellist = '<select name="filelevel" title="' . JText::_('CDATTACHMENTS_FILEACCESS') . '" size="1" id="filelevel">';
		foreach ($this->levelList() as $level) {
			if ($level->value == $fileinfo->access) {
				$levellist .= '<option value="' . $level->value . '" selected>' . $level->text . '</option>';
			} else {
				$levellist .= '<option value="' . $level->value . '">' . $level->text . '</option>';
			}
		}
		$levellist .= '</select>';

		$tmpl = '';

		$tmpl .= '<div class="fileform">';
		$tmpl .= '<form action="' . $action_url . '" name="fileform" method="post">';
		$tmpl .= '<div class="cont"><label for="filetitle">' . JText::_('CDATTACHMENTS_FILETITLE') . ': </label><input type="text" title="' . JText::_('CDATTACHMENTS_FILETITLE') . '" value="' . $fileinfo->title . '" name="filetitle" id="filetitle" maxlength="255" /></div>';
		$tmpl .= '<div class="cont"><label for="filedesc">' . JText::_('CDATTACHMENTS_FILEDESC') . ': </label><input title="' . JText::_('CDATTACHMENTS_FILEDESC') . '" type="text" name="filedesc" id="filedesc" value="' . $fileinfo->desc . '" maxlength="500" /></div>';
		$tmpl .= '<div class="cont"><label for="filepassword">' . JText::_('CDATTACHMENTS_FILEPASSWORD') . ': </label>';
		if ($mode == 'editAttachment') {
			$tmpl .= '<input title="' . JText::_('CDATTACHMENTS_FILEPASSWORD') . '" type="text" name="filepassword" id="filepassword" maxlength="255" />' . ($fileinfo->password ? $this->infoIcon(JText::_('CDATTACHMENTS_HELP_PASSWORD')) : '') . '</div>';
		} else {
			$tmpl .= '<input title="' . JText::_('CDATTACHMENTS_FILEPASSWORD') . '" type="text" name="filepassword" id="filepassword" maxlength="255" /></div>';
		}
			
		$tmpl .= '<div class="cont"><label for="filelevel">' . JText::_('CDATTACHMENTS_FILEACCESS') . ': </label>' . $levellist . '</div>';
		$tmpl .= '<div class="cont"><label for="selectfile">' . JText::_('CDATTACHMENTS_SELECTFILE') . ': </label>' . $this->selectFile($fileinfo->file) . '</div>';
		$tmpl .= '<div class="cont"><button title="' . JText::_('CDATTACHMENTS_SAVE') . '" type="submit">' . JText::_('CDATTACHMENTS_SAVE') . '</button></div>';
		if ($mode == 'addAttachment') {
			$tmpl .= '<input type="hidden" name="articleid" value="" />';
			$tmpl .= '<input type="hidden" name="attachments" value="' . $mode . '" />';
		} else {
			$tmpl .= '<input type="hidden" name="fileid" value="' . $fileinfo->id . '" />';
			$tmpl .= '<input type="hidden" name="attachments" value="' . $mode . '" />';
		}
		$tmpl .= JHTML::_( 'form.token' );
		$tmpl .= '</form>';
		$tmpl .= '</div>';

		return $tmpl;
	}

	/**
	 *
	 * @return string	Info icon image
	 */
	function infoIcon($title = '') {
		return '<img src="' . JURI::base(true) . '/plugins/content/cdattachments/images/icon_info.png" title="' . $title . '" alt="" style="margin-left: 5px; width: 16px; height: 16px;" width="16" height="16" />';
	}

	/**
	 * Save file
	 *
	 * @return void
	 */

	function addAttachment() {

		// Check for request forgeries
		JRequest::checkToken() or jexit( 'Invalid Token' );
		
		global $mainframe;
		
		if (!$this->hasEditPermission($fileid)) $this->close(JText::_('CDATTACHMENTS_NOT_ALLOWED'));
		
		$fileinfo = $this->dbFileInfo('id', $fileid);

		$file = JRequest::getString('file', '', 'post');
		$filesize = filesize($this->filePath($file));
		$filetype = $this->mimetype($file);
		$filetitle = JRequest::getString('filetitle', '', 'post');
		$filedesc = JRequest::getString('filedesc', '', 'post', 4);
		$filepassword = JRequest::getString('filepassword', '', 'post');
		$filepassword = ($filepassword ? md5($filepassword) : '');
		$filelevel = JRequest::getInt('filelevel', 0, 'post');
		$articleid = JRequest::getInt('articleid', 0, 'post');

		$user =& JFactory::getUser();

		$userid = (int) $user->id;

		$db =& JFactory::getDBO();

		$newdate =& JFactory::getDate(time(), $mainframe->getCfg('offset'));
		$editdate = $newdate = $newdate->toMySQL();
			
		$query = 'INSERT INTO `#__cdattachments` (`id`, `articleid`, `userid`, `file`, `title`, `desc`, `type`, `size`, `access`, `password`, `published`, `created`, `modified`, `modifiedby`, `downloaded`) VALUES (' .
		'0, '.
		'' . $articleid . ', '.
		'' . $userid . ', '.
		'' . $db->Quote($db->getEscaped($file)) . ', '.
		'' . $db->Quote($db->getEscaped($filetitle)) . ', '.
		'' . $db->Quote($db->getEscaped($filedesc)) . ', '.
		'' . $db->Quote($db->getEscaped($filetype)) . ', '.
		'' . $filesize . ', '.
		'' . $filelevel . ', '.
		'' . $db->Quote($db->getEscaped($filepassword)) . ', '.
		'' . 1 . ', '.
		'' . $db->Quote($newdate) . ', '.
		'' . $db->Quote($editdate) . ', '.
		'' . $userid . ', '.
		'' . 0 . ');';

		$db->setQuery($query);
		if ($db->query()) {
			$this->close();
		} else {
			$this->close(JText::_('CDATTACHMENTS_ERROR'));
		}
	}

	/**
	 * Edit file
	 *
	 * @return void
	 */

	function editAttachment() {

		// Check for request forgeries
		JRequest::checkToken() or jexit( 'Invalid Token' );
		
		global $mainframe;

		$fileid = JRequest::getInt('fileid', 0, 'post');
		
		if (!$this->hasEditPermission($fileid)) $this->close(JText::_('CDATTACHMENTS_NOT_ALLOWED'));
		
		$fileinfo = $this->dbFileInfo('id', $fileid);

		$file = JRequest::getString('file', '', 'post');
		$file = ($file ? $file : $fileinfo->file);
		$file_mime = $fileinfo->type;
		$file_size = $fileinfo->size;

		// post params
		$filetitle = JRequest::getString('filetitle', $fileinfo->title, 'post');
		$filedesc = JRequest::getString('filedesc', $fileinfo->desc, 'post', 4);
		$filepassword = JRequest::getString('filepassword', '', 'post');

		switch ($filepassword) {
			case '0':
				$filepassword = '';
				break;
			default:
				$filepassword = ($filepassword ? md5($filepassword) : $fileinfo->password);
				break;
		}

		$filelevel = JRequest::getInt('filelevel', (int) $fileinfo->access, 'post');
		$articleid =  $fileinfo->articleid;

		$newdate = $fileinfo->created;

		$editdate = & JFactory::getDate(time(), $mainframe->getCfg('offset'));
		$editdate = $editdate->toMySQL();

		$user =& JFactory::getUser();

		$userid = (int) $user->id;

		$db =& JFactory::getDBO();
		$query = 'UPDATE `#__cdattachments` SET ' .
		'`articleid` = ' . $articleid . ', '.
		'`userid` = ' . $fileinfo->userid . ', '.
		'`file` = ' . $db->Quote($db->getEscaped($file)) . ', '.
		'`title` = ' . $db->Quote($db->getEscaped($filetitle)) . ', '.
		'`desc` = ' . $db->Quote($db->getEscaped($filedesc)) . ', '.
		'`type` = ' . $db->Quote($db->getEscaped($file_mime)) . ', '.
		'`size` = ' . $file_size . ', '.
		'`access` = ' . $filelevel . ', '.
		'`password` = ' . $db->Quote($db->getEscaped($filepassword)) . ', '.
		'`published` = ' . 1 . ', '.
		'`created` = ' . $db->Quote($newdate) . ', '.
		'`modified` = ' . $db->Quote($editdate) . ', '.
		'`modifiedby` = ' . $userid . ', '.
		'`downloaded` = ' . $fileinfo->downloaded . ' WHERE `id` = ' . $fileinfo->id . ';';

		$db->setQuery($query);
		if (!$db->query()) {
			$this->close(JText::_('CDATTACHMENTS_UPLOAD_ERROR'));
				
		} else {
			$this->close();
		}

	}

	/**
	 * File management
	 *
	 * @return string
	 */
	function filemgmt() {
		
		if (!$this->filemanagerPermission()) return JText::_('CDATTACHMENTS_NOT_ALLOWED');

		$files = JFolder::files($this->attachmentsDir(), '.', false, false, array('.svn', 'CVS', 'index.html', '.htaccess'));
		$html = '<div class="filemanager">';
			$html .= '<div class="removefile">';
				$html .= '<div>' . JText::_('CDATTACHMENTS_REMOVEFILE') . '</div>';
					$html .= '<form action="' . $this->url() . '" name="removefile" method="post">';
						$html .= '<select name="file" title="' . JText::_('CDATTACHMENTS_SELECTFILE') . '" multiple size="10">';
						foreach ($files as $file_item) {
							$html .= '<option value="' . $file_item . '">' . $file_item . '</option>';
						}
						$html .= '</select>';
						$html .= '<div style="margin-top: 5px; text-align: right;"><button title="' . JText::_('CDATTACHMENTS_REMOVEFILE') . '" type="submit">' . JText::_('CDATTACHMENTS_REMOVEFILE') . '</button></div>';
						$html .= JHTML::_( 'form.token' );
						$html .= '<input type="hidden" name="attachments" value="removeFile">';
					$html .= '</form>';
			$html .= '</div>';
			$html .= '<div class="uploadfile">';
				$html .= '<div>' . JText::_('CDATTACHMENTS_UPLOAD') . '</div>';
					$html .= '<form action="' . $this->url() . '" enctype="multipart/form-data" name="uploadfile" method="post"><input title="' . JText::_('CDATTACHMENTS_BROWSE') . '" name="file" type="file" />';
						$html .= '<div style="margin-top: 5px; text-align: right;"><button title="' . JText::_('CDATTACHMENTS_UPLOAD') . '" type="submit">' . JText::_('CDATTACHMENTS_UPLOAD') . '</button></div>';
						$html .= JHTML::_( 'form.token' );
						$html .= '<input type="hidden" name="attachments" value="uploadFile">';
					$html .= '</form>';
			$html .= '</div>';

		$html .= '</div>';

		return $html;
	}

	/**
	 * Upload file function
	 * @return void
	 */
	function uploadFile() {

		// Check for request forgeries
		JRequest::checkToken() or jexit( 'Invalid Token' );
		
		global $mainframe;
		
		if (!$this->filemanagerPermission()) $mainframe->redirect($this->url(), (JText::_('CDATTACHMENTS_NOPERMISSIONTOUPLOAD')));

		if (!isset($_FILES['file']) or $_FILES['file']['error']) {
			$mainframe->redirect($this->url(), JText::_('CDATTACHMENTS_UPLOAD_ERROR'));
		}

		$file_uploadname = $_FILES['file']['name'];
		$file_tmp_path = $_FILES['file']['tmp_name'];
		$file_mime = $_FILES['file']['type'];
		$file_size = $_FILES['file']['size'];

		// file not allowed to upload
		$allowedToUpload = $this->allowedToUpload($_FILES);
		if ($allowedToUpload) {
			$mainframe->redirect($this->url(), $allowedToUpload);
		}

		// file already exists
		if (JFile::exists($this->filePath($file_uploadname))) {
			$mainframe->redirect($this->url(), JText::sprintf('CDATTACHMENTS_FILE_ALREADY_EXISTS', $file_uploadname));
		}

		if(JFile::upload($file_tmp_path, $this->filePath($file_uploadname))) {
			// file have been uploaded
			$mainframe->redirect($this->url(), JText::sprintf('CDATTACHMENTS_FILE_UPLOADED', $file_uploadname));
		} else {
			// uploading error
			$mainframe->redirect($this->url(), JText::_('CDATTACHMENTS_UPLOAD_ERROR'));
		}
	}

	/**
	 * Remove rfile
	 *
	 * @return string
	 */
	function removeFile() {

		// Check for request forgeries
		JRequest::checkToken() or jexit( 'Invalid Token' );

		if (!$this->filemanagerPermission()) $this->close(JText::_('CDATTACHMENTS_NOPERMISSIONTODELETE'));
		
		$file = explode(',', JRequest::getString('file', '', 'post'));

		$msg = null;
		foreach ($file as $file_item) {
			if (JFile::delete($this->attachmentsDir() . DS  . $file_item) and $file_item) {
				$msg .= JText::sprintf('CDATTACHMENTS_FILE_DELETED', $file_item) . '<br />';
			} else {
				$msg .= JText::sprintf('CDATTACHMENTS_FILE_DELETE_ERROR', $file_item) . '<br />';
			}
		}

		$this->close($msg);

	}

	/**
	 * Delete file
	 *
	 * @return string	Empty if success, text if error.
	 */
	function deleteAttachment() {

		// Check for request forgeries
		JRequest::checkToken() or jexit( 'Invalid Token' );

		$fileid = JRequest::getInt('fileid', 0, 'post');

		$fileinfo = $this->dbFileInfo('id', $fileid);

		if (!$this->hasEditPermission($fileid)) return JText::_('CDATTACHMENTS_NOPERMISSIONTODELETE');

		$db = &JFactory::getDBO();

		$query = 'DELETE FROM `#__cdattachments` WHERE `id` = ' . $fileid . '';

		$db->setQuery($query);

		if ($db->query()) {
			return '';
		} else {
			return JText::_('CDATTACHMENTS_ERROR');
		}
	}

	/**
	 * Check if user is authorized to delete the file, file's author by default
	 *
	 * @param $articleid	int
	 *
	 * @return boolean
	 */
	function hasEditPermission($fileid = 0) {

	 $user =& JFactory::getUser();
	 $userid = (int) $user->id;
	 $gid = $user->gid;

	 if ($user->guest) return false; // disable for guests
	 if ($user->gid == '25') return true; // enable for superadministrator accounts
	 
	 $addnew_allowed = $this->params->get('addnew_restriction');
	 
	 // edit file if I'm file author
	 $file = $this->dbFileInfo('id', $fileid);
	 if ($file) {
	 	if (is_array($addnew_allowed) and $userid == $file->userid and in_array($gid, $addnew_allowed)) return true;
	 	if (!is_array($addnew_allowed) and $userid == $file->userid and $addnew_allowed == $gid or $gid == '25') return true;
	 	return false;
	 }
	 
	 if (is_array($addnew_allowed) and in_array($gid, $addnew_allowed)) return true;
	 if (!is_array($addnew_allowed) and $addnew_allowed == $gid or $gid == '25') return true;

	 return false;
	}
	
	/**
	 * Check if user has permission to upload the files
	 * 
	 * @return boolean
	 */
	function filemanagerPermission() {

		$user =& JFactory::getUser();
		$gid = $user->gid;
		if ($user->guest) return false; // disable for guests
		if ($gid == '25') return true; // enable for superadministrator accounts
		
		$filemanagement_allowed = $this->params->get('filemanager_restriction');
		
		// level array
		if (is_array($filemanagement_allowed) and in_array($gid, $filemanagement_allowed)) return true;
		
		// level string
		if (!is_array($filemanagement_allowed) and $filemanagement_allowed == $gid) return true;
		return false;
	}

	/**
	 * Check if filename is allowed to upload
	 *
	 * @param $file
	 *
	 * @return string	Error message. if empty, file is correct.
	 */
	function allowedToUpload($file = null) {

		if (!$file) return JText::_('CDATTACHMENTS_UPLOAD_ERROR');

		$file_uploadname = $_FILES['file']['name'];
		$file_mime = $_FILES['file']['type'];
		$file_size = $_FILES['file']['size'];

		if ($file_size > ((int)$this->params->get('reject_filesize' , 10) * 1000000)) {
			return JText::sprintf('CDATTACHMENTS_MAX_SIZE', (int)$this->params->get('reject_filesize' , 10));
		}

		$reject_filename = array_map('trim', explode("\n", trim($this->params->get('reject_filename' , ''))));
		array_push($reject_filename, '.htaccess', 'index.html', 'index.htm', 'index.php');

		$reject_ext = array_map('trim', explode("\n", trim($this->params->get('reject_ext' , ''))));
		if (in_array($file_uploadname, $reject_filename) or in_array(JFile::getExt($file_uploadname), $reject_ext)) {
			return JText::sprintf('CDATTACHMENTS_NOTALLOWED_TO_UPLOAD', $file_uploadname);
		}

		return '';
	}

	/**
	 * Atttachment directory
	 *
	 * Return absolute path to the attachment directory.
	 *
	 * @return string
	 */
	function attachmentsDir() {
		$abs_path = dirname(dirname(dirname(__FILE__)));
		$dir = JPath::clean($abs_path . DS . $this->params->get('directory', 'attachments'));

		return $dir;
	}

	/**
	 * Path to the file
	 *
	 * Return absolute path to the required file
	 *
	 * @param	$filename	filename
	 *
	 * @return string
	 */
	function filePath($filename = '') {
		if ($filename) {
			$path = JPath::clean($this->attachmentsDir() . DS . $filename);
		} else {
			$path = '';
		}
		return  $path;
	}

	/**
	 * Present a size (in bytes) as a human-readable value
	 *
	 * @param int    $bytes        size (in bytes)
	 *
	 * @return string
	 */
	function byte_convert($bytes = 0) {
		$symbol = array('B', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'ZiB', 'YiB');

		$exp = 0;
		$converted_value = 0;
		if( $bytes > 0 )
		{
			$exp = floor( log($bytes)/log(1024) );
			$converted_value = ( $bytes/pow(1024,floor($exp)) );
		}

		return sprintf( '%.2f '.$symbol[$exp], $converted_value );
	}

	/**
	 * Get Joomla! levels
	 *
	 * @return array	Joomla! levels
	 */
	function levelList() {

		$db =& JFactory::getDBO();
		$query = 'SELECT id AS value, name AS text'
		. ' FROM #__groups'
		. ' ORDER BY id'
		;
		$db->setQuery( $query );
		$groups = $db->loadObjectList();
		
		$user =& JFactory::getUser();
		$aid = $user->aid;
		
		$groups_re = array();
		foreach($groups as $item) {
			if ($item->value <= $aid) {
				$groups_re []= $item;
			}
		}
		
		return $groups_re;

	}

	/**
	 * Find mime type
	 *
	 * @param $filename
	 * @return string	Mime type
	 */
	function mimetype($filename = '') {
		require_once (dirname(__file__) . DS . 'cdattachments' . DS . 'utils' . DS . 'php' . DS . 'mime_class.php');

		$mimetype = new cdAttachmentsHelperMimeType(); // load mime type
		$mimetype = $mimetype->getType($filename); // get mime type

		return $mimetype;
	}

	/**
	 * Download file
	 *
	 * @param $filepath		Path to the file.
	 *
	 * @return void
	 */
	function download() {

		// Check for request forgeries
		JRequest::checkToken() or jexit( 'Invalid Token' );

		global $mainframe;

		$fileid = JRequest::getInt('fileid', 0, 'post');

		$fileinfo = $this->dbFileInfo('id', $fileid);
		
		if (!$fileinfo) {
			$mainframe->redirect($this->url(), JText::_('CDATTACHMENTS_ERROR'));
		}

		$filename = $fileinfo->file;

		$filepath = $this->filePath($filename);
		
		if (!JFile::exists($filepath)) $mainframe->redirect($this->url(), JText::sprintf('CDATTACHMENTS_FILE_NOT_EXISTS', $filename));

		// check if file is procted via password
		if ($fileinfo->password) {
			$filepassword = JRequest::getString('filepassword', '', 'post');
				
			if (!$filepassword or md5($filepassword) != $fileinfo->password) {
				$mainframe->redirect($this->url(), JText::_('CDATTACHMENTS_PASSWORD_INCORRECT'));
			}
		}

		// load required classes
		require_once (dirname(__file__) . DS . 'cdattachments' . DS . 'utils' . DS . 'php' . DS . 'download_class.php');

		$filename = basename($filepath); // filename
		$mimetype = $this->mimetype($filename); // get mime type

		$filepath = new DOWNLOADFILE($filepath, $mimetype, 'attachment', $filename); // load file to download

		if (!$filepath->df_download())
		{
			$mainframe->redirect($this->url(), JText::_('CDATTACHMENTS_ERROR'));
		} else
		{
			$this->hit($fileid); // hit
			$this->close();
		}
	}

	/**
	 * Hit attachment if download
	 *
	 * @param $id	Attachment ID
	 *
	 * @return boolean
	 */
	function hit($id = 0) {

		$fileinfo = $this->dbFileInfo('id', $id);

		$db =& JFactory::getDBO();
		$query = 'UPDATE `#__cdattachments` SET ' .
		'`downloaded` = ' . ($fileinfo->downloaded + 1) . ' WHERE `id` = ' . $id . ';';

		$db->setQuery($query);
		if (!$db->query()) {
			return false;
		} else {
			return true;
		}
	}

	/**
	 * Get actual URL
	 *
	 * @return string	URL
	 */
	function url() {
		jimport('joomla.enviroment.request');
		return JRequest::getURI();
	}

	/**
	 * Get file information from database
	 *
	 * @param $key
	 * @param $val
	 *
	 * @return object
	 */
	function dbFileInfo($key = 'id', $val = 0) {
		if (!$val or !$key) return null;

		$db = &JFactory::getDBO();
		$query = 'SELECT * FROM `#__cdattachments` WHERE `' . $key . '` = ' . $val . '';

		$db->setQuery($query);
		$row = $db->loadObject();

		return $row;
	}
}
?>