/**
 * Application - Attachments
 */

(function($) {
	$.attachments = {
		
		// Init application
		initiator: function() {
			$.attachments.buttonsUI($('.attachments'));
			
			// load attachments
			$('.attachments .loadattachments').click(function() {
				articleid = $(this).attr('id'); // global article id variable
				$.attachments.enableDialog($(this));
			});
			
		},
		
		// Enable dialog
		enableDialog: function(clickable) {
			
			// prevent category and section blog preview
			var enableDialog = $('body > .' + $.attachments.settings.theme + '');
			if (enableDialog.length) return false;
			
			$('<div id="attachments"></div>').appendTo('body');
			var attachment = $('#attachments');
			
			attachment.dialog({
				title: $.attachments.language.CDATTACHMENTS,
				height: 400,
				width: 600,
				draggable: false,
				resizeable: false,
				position: 'center',
				stack: false,
				buttons: {
					'close' : function() {
						$(this).dialog('close');
					},
					'files' : function() {
						$.attachments.filemgmt(attachment);
					},
					'addnew' : function() {
						$.attachments.addnew(attachment);
					},
					'back' : function() {
						$.attachments.load(attachment);
					}
				},
				open: function() {
					clickable.attr('disabled', true).addClass('ui-state-disabled'); // disable Attachments button
					$.attachments.theme($(this)); // add theme
					$.attachments.load($(this)); // load attachments
					$.attachments.buttonTitles(); // button routine
					$.attachments.buttonRoutine(); // hide back button
				},
				close: function() {
					clickable.attr('disabled', false).removeClass('ui-state-disabled');
					$.attachments.removeDialog($(this));
					$('body > #attachments').remove(); // remove container
				}
			});
			
		},
		
		// Load attachments
		load: function(container) {
			
			$.ajax({
				type: 'POST',				
				dataType: 'html',
				timeout: 8000,
				cache: true,
				data: 'attachments=getItems' + '&articleid=' + articleid + '&' + $.attachments.settings.token + '=1',
				beforeSend: function() {
					$.attachments.loading();
					container.dialog('disable');
				},
				success: function(msg) {
					
					container.html(msg);
					$.attachments.buttonsUI(container); // add UI theme to buttons in container
					$.attachments.fileinformationicon(container); // file info icon routine
					$.attachments.deleteIcon(container); // delete icon routine
					$.attachments.editIcon(container); // edit icon routine
					
					var form = container.find('form[name="downloadfile"]');
					form.submit(function(e) {
						
						// check password
						var password = $(this).find('input[name="filepassword"]:hidden');
						
						if (!password.length) return true;
						
						var pass = $.trim(prompt($.attachments.language.CDATTACHMENTS_FILE_IS_PROTECTED, ''));
						if (pass != null && pass != '') {
							password.val(pass);
						} else {
							e.preventDefault();
							return false;
						}
						
					});	
					
				},
				error: function() {
					alert($.attachments.language.CDATTACHMENTS_ERROR);
					return false;
				},
				complete: function() {
					$.attachments.buttonRoutine();
					$.attachments.loading();
					container.dialog('enable');
				}
			});
		},
		
		// File manager
		filemgmt: function(container) {
			$.ajax({
				type: 'POST',				
				dataType: 'html',
				timeout: 8000,
				cache: true,
				data: 'attachments=filemgmt&' + $.attachments.settings.token + '=1',
				beforeSend: function() {
					$.attachments.loading();
					container.dialog('disable');
				},
				success: function(msg) {
					$.attachments.buttonRoutine('filemgmt');
					container.html(msg);
					$.attachments.buttonsUI(container); // add UI theme to buttons in container
					
					// save upload button
					container.find('form[name="removefile"]').submit(function (e) {
						e.preventDefault();
						if ($(this).find('select[name="file"]').val()) {
							$.ajax({
								type: 'POST',				
								dataType: 'html',
								timeout: 8000,
								cache: true,
								data: 'attachments=removeFile' + '&file=' + $(this).find('select[name="file"]').val() + '&' + $.attachments.settings.token + '=1',
								beforeSend: function() {
									$.attachments.loading();
									container.dialog('disable');
								},
								success: function(msg) {
									$.attachments.buttonRoutine('files');
									container.html(msg);
								},
								error: function() {
									alert($.attachments.language.CDATTACHMENTS_ERROR);
									return false;
								},
								complete: function() {
									$.attachments.loading();
									container.dialog('enable');
								}
							});
						} else {
							alert($.attachments.language.CDATTACHMENTS_SELECT_FILE);
							return false;
						}
					});
					
					// upload button
					container.find('form[name="uploadfile"]').submit(function (e) {
						if ($(this).find('input[name="file"]').val()) {
							return true;
						} else {
							alert($.attachments.language.CDATTACHMENTS_SELECT_FILE);
							e.preventDefault();
							return false;
						}
					});
				},
				error: function() {
					alert($.attachments.language.CDATTACHMENTS_ERROR);
					return false;
				},
				complete: function() {
					$.attachments.loading();
					container.dialog('enable');
				}
			});
		},
		
		// Add new
		addnew: function(container) {
			
			$.ajax({
				type: 'POST',				
				dataType: 'html',
				timeout: 8000,
				cache: true,
				data: 'attachments=fileTemplate' + '&' + $.attachments.settings.token + '=1',
				beforeSend: function() {
					$.attachments.loading();
					container.dialog('disable');
				},
				success: function(msg) {
					
					$.attachments.buttonRoutine('addnew');
					
					container.html(msg);
					$.attachments.buttonsUI(container);
					
					var form = container.find('form[name="fileform"]');
					
					form.find('input[name="filetitle"]').focus();
					
					form.submit(function(e) {
						
						e.preventDefault();
						
						var emptylang = $.attachments.language.CDATTACHMENTS_EMPTY_VALUE;
						
						var filetitle = $(this).find('input[name="filetitle"]');
						if (filetitle.val() == '') {
							alert(emptylang);
							filetitle.focus();
							return false;
						}
						
						var file = $(this).find('select[name="file"]');
						if (file.val() == '') {
							alert(emptylang);
							file.focus();
							return false;
						}
						
						$(this).find('input[name="articleid"]:hidden').val(articleid); // insert article id to hidden input
						
						$.ajax({
							type: 'POST',	
							dataType: 'html',
							timeout: 8000,
							cache: true,
							data: $(this).serialize(),
							beforeSend: function() {
								$.attachments.loading();
								container.dialog('disable');
							},
							success: function(msg) {
								if (msg) {
									alert(msg);
									return false;
								} else {
									var count = $('button[id="' + articleid + '"]').children('span');
									count.text(count.text() * 1 + 1);
									
									$.attachments.successInfo(container, $.attachments.language.CDATTACHMENTS_SAVED);
									$.attachments.buttonRoutine('success');
								}
							},
							error: function() {
								alert($.attachments.language.CDATTACHMENTS_ERROR);
								return false;
							},
							complete: function() {
								$.attachments.loading();
								container.dialog('enable');
							}
						});
						
					});
					
				},
				error: function() {
					alert($.attachments.language.CDATTACHMENTS_ERROR);
					return false;
				},
				complete: function() {
					$.attachments.loading();
					container.dialog('enable');
				}
			});
		},
		
		// Activate delete icon routine
		deleteIcon: function(container) {
			container.find('.document .fileicons .fileicondelete').click(function() {
				
				var really = confirm($.attachments.language.CDATTACHMENTS_REALLY);
				if (!really) return false;
				
				var document = $(this).parent('.fileicons').parent('.document');
				var fileid = document.find('.downloadbutton').find('input[name="fileid"]:hidden').val();
				
				$.ajax({
					type: 'POST',				
					dataType: 'html',
					timeout: 8000,
					cache: true,
					data: 'attachments=deleteAttachment' + '&fileid=' + fileid + '&' + $.attachments.settings.token + '=1',
					beforeSend: function() {
						container.dialog('disable');
						$.attachments.loading();
					},
					success: function(msg) {
						if (msg) {
							alert(msg);
							return false;
						} else {
							$.attachments.successInfo(container, $.attachments.language.CDATTACHMENTS_DELETED);
							
							var count = $('button[id="' + articleid + '"]').children('span');
							count.text(count.text() * 1 - 1);
							$.attachments.buttonRoutine('success');
							
							$.attachments.buttonRoutine('success');
						}
					},
					error: function() {
						alert($.attachments.language.CDATTACHMENTS_ERROR);
						return false;
					},
					complete: function() {
						container.dialog('enable');
						$.attachments.loading();
					}
				});
				
			});
		},
		
		// Activate edit icon function
		editIcon: function(container) {
			
			container.find('.document .fileicons .fileiconedit').click(function() {
				
				var document = $(this).parent('.fileicons').parent('.document');
				var fileid = document.find('.downloadbutton').find('input[name="fileid"]:hidden').val();
				
				$.ajax({
					type: 'POST',				
					dataType: 'html',
					timeout: 8000,
					cache: true,
					data: 'attachments=fileTemplate' + '&fileid=' + fileid + '&' + $.attachments.settings.token + '=1',
					beforeSend: function() {
						$.attachments.loading();
						container.dialog('disable');
					},
					success: function(msg) {
						
						$.attachments.buttonRoutine('edit');
						
						container.html(msg);
						
						$.attachments.buttonsUI(container);
						
						var form = container.find('form[name="fileform"]');
						
						form.find('input[name="filetitle"]').focus();
						
						form.submit(function(e) {
							
							e.preventDefault();
							
							var emptylang = $.attachments.language.CDATTACHMENTS_EMPTY_VALUE;
							
							var filetitle = $(this).find('input[name="filetitle"]');
							if (filetitle.val() == '') {
								alert(emptylang);
								filetitle.focus();
								return false;
							}
							
							var file = $(this).find('select[name="file"]');
							if (file.val() == '') {
								alert(emptylang);
								file.focus();
								return false;
							}
							
							$.ajax({
								type: 'POST',				
								dataType: 'html',
								timeout: 8000,
								cache: true,
								data: $(this).serialize(),
								beforeSend: function() {
									$.attachments.loading();
									container.dialog('disable');
								},
								success: function(msg) {
									if (msg) {
										alert(msg);
										return false;
									} else {
										$.attachments.successInfo(container, $.attachments.language.CDATTACHMENTS_EDITED);
										$.attachments.buttonRoutine('success');
									}
								},
								error: function() {
									alert($.attachments.language.CDATTACHMENTS_ERROR);
									return false;
								},
								complete: function() {
									$.attachments.loading();
									container.dialog('enable');
								}
							});
							
						});
						
					},
					error: function() {
						alert($.attachments.language.CDATTACHMENTS_ERROR);
						return false;
					},
					complete: function() {
						$.attachments.loading();
						container.dialog('enable');
					}
				});
			});
		},
		
		// Activate file information icon
		fileinformationicon: function(element) {
			if (!element.length) return false;
			
			var isDialogOpen = false;
			
			element.find('.document .fileicons .fileiconinfo').click(function() {
				
				if (isDialogOpen) return true; // prevent double dialog calling
				
				var fileinfo = $(this).parent('.fileicons').parent('.document').find('.fileinformation').clone();
				
				var panelbuttons = element.next('div.ui-dialog-buttonpane').children('button');
				
				fileinfo.dialog({
					title: $.attachments.language.CDATTACHMENTS_FILEINFORMATION,
					height: 150,
					width: 350,
					draggable: false,
					resizeable: false,
					position: 'center',
					stack: true,
					buttons: {
						'close' : function() {
							$(this).dialog('close');
						}
					},
					open: function() {
						element.dialog('disable');
						panelbuttons.attr('disabled', true).addClass('ui-state-disabled');
						$.attachments.buttonTitles(); // button routine
						$.attachments.theme($(this));
						isDialogOpen = true;
					},
					close: function() {
						element.dialog('enable');
						panelbuttons.attr('disabled', false).removeClass('ui-state-disabled');
						isDialogOpen = false;
						$.attachments.removeDialog($(this));
						$('body > .fileinformation').remove(); // remove file information container
					}
				});
			});
		},
		
		// Loading wrapper
		loading: function(container) {
			var buttonpanel = $('.ui-dialog .ui-dialog-buttonpane');
			
			if (buttonpanel.children('.attachments_loading').length) {
				buttonpanel.children('.attachments_loading').remove();
				buttonpanel.find('button').attr('disabled', false).removeClass('ui-state-disabled');
			} else {
				buttonpanel.find('button').attr('disabled', true).addClass('ui-state-disabled');
				buttonpanel.prepend('<span class="attachments_loading"></span>');
			}
			
		},
		
		// Add to buttons the UI theme
		buttonsUI: function(container) {
			
			var buttons = container.find('button');
			
			buttons.addClass('ui-state-default ui-corner-all');
			
			// button is disabled
			if (buttons.is(':disabled')) {
				container.find('button').addClass('ui-state-disabled');
				return false;
			}
			
			buttons.hover(
				function(){ 
					$(this).addClass('ui-state-hover');
					$(this).click(function() {
						$(this).removeClass('ui-state-hover');
					});
				},
				function(){ 
					$(this).removeClass('ui-state-hover'); 
				}
			);
		},
		
		// Wrapper dialog with related theme
		theme: function(container) {	
			var wrapper = '<div class="' + $.attachments.settings.theme + '"></div>';
			container.parent('.ui-dialog').wrap(wrapper);
			$('.ui-widget-overlay').wrap(wrapper); // if overlay
		},
		
		// Destroy dialog
		removeDialog: function(container) {
			container.parent('.ui-dialog').parent('.' + $.attachments.settings.theme + '').remove();
		},
		
		// Manage button displaying based on layout
		buttonRoutine: function(layout) {
			var buttons = $('.ui-dialog .ui-dialog-buttonpane');
			switch (layout) {
				case 'addnew':
				case 'edit':
				case 'success':
				case 'filemgmt':
					buttons.find('.back').show();
					buttons.find('.addnew, .files').hide();
					break;
				case 'files':
					buttons.find('.files').show();
					buttons.find('.back, .addnew').hide();
					break;
				default:
					buttons.find('.back').hide();
					buttons.find('.addnew, .files').show();
					break;
			}
			
		},
		
		// Display buttons routine
		buttonTitles: function() {
			var buttons = $('.ui-dialog .ui-dialog-buttonpane button');
			buttons.each(function() {
				switch($(this).text()) {
					case 'close':
						$(this).text($.attachments.language.CDATTACHMENTS_CLOSE).attr('title', $.attachments.language.CDATTACHMENTS_CLOSE);
						break;
					case 'addnew':
						if ($.attachments.settings.addnewauth != '1') {
							$(this).remove();
							return;
						}
						$(this).text($.attachments.language.CDATTACHMENTS_ADDNEW).attr('title', $.attachments.language.CDATTACHMENTS_ADDNEW).addClass('addnew');
						break;
					case 'files':
						if ($.attachments.settings.filemanagerauth != '1') {
							$(this).remove();
							return;
						}
						$(this).text($.attachments.language.CDATTACHMENTS_FILES).attr('title', $.attachments.language.CDATTACHMENTS_FILES).addClass('files');
						break;
					case 'back':
						$(this).text($.attachments.language.CDATTACHMENTS_BACK).attr('title', $.attachments.language.CDATTACHMENTS_BACK).addClass('back');
						break;
					default:
						break;
				}
			});
		},
		
		// Success information
		successInfo: function(container, msg) {
			container.html($('<div id="attachments_success"><p><span style="margin: 0pt 7px 50px 0pt; float: left;" class="ui-icon ui-icon-check"/></span>' + msg + '</p></div>'));
		}
		
	};
})(jQuery);